program OneLayerWell_scenariosAug
!
! The original program singlelayerwell, file singlelayerwell.f90, was written by Steve Larson, SSPA.

! This is a variation on the multilayer version; layer is set to 1 in two places in code.

! File OneLayerWell_scenarios.f90 is a modified version by Sam Perkins, KDA-DWR (scenarios 1-8).
!                                    --modified 9/24/2014 by Sam to specify fraction of pumping within masked zone instead of complete shutoff.
!                                    --modified 9/26/2014 by Ginger Pugh, KDA-DWR, to add scenarios 9-11

! 10/16/2014 spp: cleaned up scenario well selection for multilayer version of preprocessor:
!                 disabled statement "LAY = 1" in parameter section of pumping file preceding stress periods
!                 reinstated "incl_well = 0" technique of selecting out wells for scenario, but taking into account pumpfactor with "elseif" clause.
! 10/21/2014 spp: set up single-layer version of the multilayerwell_scenarios program by setting LAY = 1 in two places in code.

! INPUT:

! OUTPUT:

! Program createsinglelayerwell translates a multilayer well input file produced by Balleau
! into a single-layer well input file.

! Original procedure outline for createsinglelayerwell:
!   1. Read names of multilayer input well file and single-layer output well file from text file SINGLELAYERWELL.IN.
!   2. Open input and output well files, specif. multilayer file bbgmdmod.wel developed by Balleau and
!      single-layer output file BBGMDMOD_V6_1LAYER.WEL.
!   3. Read the multilayer well file and, line by line, write a single-layer version.
!      a. For each well with input data {Lay Row Col Q} set Lay = 1 before writing data to output file.
!   End
!
! Source code language notes:
!   1. Source code is Fortran-90 free format, compiled at KDA-DWR first with Lahey-95 Express v. 5.6, but currently with Gnu Fortran.
!   2. The statement IMPLICIT NONE requires that all variables must be declared explicitly;
!   3. The modified code is Fortran-90 free format and retains the IMPLICIT NONE requirement.

! Compiling with Gnu Fortran:
! Prior to compiling, linking or running:
!   set path=%path%;C:\MinGW\bin\
! To compile:
!   Switches: -c: Compile without linking; -o [specify executable filename]; -ffree-line-length-none (free format source code, no line length limit), -Wall (Warnings all)
!   gfortran -c -Wall -Wextra -fcheck=bounds -Fbacktrace -ffree-line-length-none OneLayerWell_scenariosAug.f90
!   gfortran -static -o OneLayerWell_scenariosAug OneLayerWell_scenariosAug.o

! Added features of program OneLayerWell_scenarios:
!   1. Read input filename as command line argument (see example for baseline above)

!   2. Build in discretization consistent with file bbgmdmod_1Layer.DIS to relate stress period to year and month
!      and convert flow rate (cubic feet per day) to volume (acre-feet).

!   3. Read comma-delimited text file UpStreamGrid_CellCenters_Ident.csv for entire upstreamGrid extent (180 rows by 335 columns)
!      For each grid cell, read:
!        Node number and associated (row,column) indices;
!        Grid cell center (x,y) coordinates of model projection (KS State Plane South, NAD 1983, U.S. feet);
!        Land surface elevation;
!        Ibound array to define active model domain;
!        GMD5 zone identifier ingmd5 (1 within GMD5, 0 outside GMD5)
!        DWR basin identifier dwrbsn (50 for Rattlesnake C)

!      Note: File UpStreamGrid_CellCenters_Ident.csv was created by importing the ~.dbf component of shapefile ~.shp into Excel
!      and exporting it in the csv format. The procedure for producing the shapefile may deserve summarizing. It was done using
!      ArcView 3.2 with the projection utility and Xtools extension. The Identity operation available with Xtools was applied to associate
!      DWR subbasin zones with model grid cell centers. [The Identity operation is available in ArcGIS, but only with the intermediate or
!      advanced license level, as I discovered after investigating in April or May. I have a beginner ArcGIS version, but Tom Morey, DWR,
!      at one time an interim ArcGIS manager, said he found me on a list of those slated for the intermediate version.]

!   4. Accumulate annual pumping (acre-feet) in array Qacftyr.
!      Retain annual pumping for calendar year 1957 in array Qacftyr_1957.

!   5. Interpret pumping (Q, positive or negative) in well file to be associated with water rights junior to 1957 (year of USF&W Quivira rights)
!      if calendar year > 1957 and negligible pumping occurred in corresponding cell in 1957; i.e.
!      if (iyr.gt.1957.and.abs(Qacftyr_1957(col,row)).lt.1.)

!   6. Accumulate annual sums over model domain and write sums to std output (redirected as log file e.g. gmd5_wells_scenario_1.log).
!      Write to log file for each year:
!         i,iyr,sumPosGMD5,sumGMD5,sumGMD5_jr,nwell_Rsnake,sumPosRsnake,sumRsnake,sumRsnake_jr,sumPos,sumNeg,sumAll,sumIncl,sumExcl
!           i, iyr:       ending stress period for year and calendar year;
!           sumPosGMD5:   total injected water within GMD5 (Q is positive, i.e. water into aquifer);
!           sumGMD5:      sum over all pumping within GMD5 (both positive and negative Q);
!           sumGMD5_jr:      sum over all pumping within GMD5 that is interpreted as junior to USF&W Quivira water rights (see 5, above);
!           nwell_Rsnake: no. "wells" within Rattlesnake basin, including both pumping (negative Q) and return flow (positive Q);
!           sumPosRsnake: sum over positive Q within Rattlesnake basin (return flow)
!           sumRsnake:    sum over all Q within Rattlesnake basin
!                         [total pumping, i.e. negative Q, in Rattlesnake basin: sumNegRsnake = sumRsnake - sumPosRsnake, calculated in Excel]
!           sumRsnake_jr: sum over all pumping within Rattlesnake basin that is interpreted as junior to USF&W Quivira water rights (see 5, above);
!           sumPos:       total injected water over full model extent (Q is positive, i.e. water into aquifer)
!                         (includes both irrigation return flow and boundary condition flows);
!           sumNeg:       total extracted water over full model extent (Q is negative, i.e. water out of aquifer);
!           sumAll:       sum over all pumping (all "wells") within model extent (both positive and negative Q);
!           sumIncl:      sum over all included pumping within model extent that is not excluded by scenario conditions (see 7, below);
!           sumExcl:      sum over all pumping excluded by scenario conditions (see 8, below).

!   7. All pumping within Rattlesnake C basin is written (not only to the specified output well file but also) to file rattlesnake.wel
!      for each stress period, along with the same stress period header as the one written to the specified output well file.
!      The output written to file Rattlesnake.wel for June 1954 was imported into file ss_hist_gmd5_pumping.xlsx, sheet rattlesnake.wel_Jun_1954,
!      and plotted. The chart at I3 shows the pumping locations in terms of model grid (col,row) coordinates as two separate groups from range b2:c128
!      of negative (red) pumping at 27 locations (b2:c28) and positive (blue) return flow at 110 locations (b29:c128). The ratio of total return flow to
!      pumping is 52 percent. Interpreting this as irrigation return flow, the denominator may be exaggerated by inclusion of nonirrigation pumping,
!      for which return flow is not represented (modeled as zero) in the Balleau model.

!   8. Define scenarios that can be specified by variable id_scenario (and summarized by text inch_scenario) in the parameter input file:
!      0:  baseline: output well file is same as input well file.
!      1:  Exclude all junior pumping within Rattlesnake C basin after 1957. (see gmd5_wells_scenario_1.IN)
!      1: shut off all Rattlesnake C bsn wells junior to Aug 15 1957 (USF&W File 7571) beginning with str period 218 (Jan 1958)
!      2: shut off all Rattlesnake C bsn wells junior to Aug 15 1957 (USF&W File 7571) beginning with str period 602 (Jan 1990)
!      3: shut off RSC bsn junior wells within a 1mi corridor along Rattlesnake C above Quivira intake
!      4: shut off RSC bsn junior wells within alluvial extent of Rattlesnake C above Quivira intake
!      5: shut off RSC bsn junior wells within georeferenced Balleau stream flow rapid response zone >70% above Quivira intake
!      6: shut off RSC bsn junior wells within georeferenced Balleau stream flow rapid response zone 40%-70% above Quivira intake
!      7: shut off RSC bsn junior wells within Balleau stream flow rapid response zone 70%-100% above Quivira intake
!      8: shut off RSC bsn junior wells within georeferenced Balleau stream flow rapid response zone 40%-100% above Quivira intake
!      9: shut off RSC bsn junior wells within georeferenced Balleau stream flow rapid response zone 20%-100% above Quivira intake
!      10: shut off RSC bsn Junior wells within a 1mi corridor along Rattlesnake C from the Macksville station to Quivira intake
!      11: shut off RSC bsn Junior wells within a 2mi corridor along Rattlesnake C from the Macksville station to Quivira intake
!
!   9. To run scenarios after invoking a DOS window and navigating to \\ag-DWR\Files\HydroModel\GMD5\1L\test>
!      baseline (without redirecting std I/O):
!          OneLayerWell_scenarios
!      scenario 1 (redirecting both stdin and stdout):
!          OneLayerWell_scenarios < gmd5_wells_scenario_1.rsp > gmd5_wells_scenario_1.log
!          Redirected output: the summary in file gmd5_wells_scenario_1.log was imported into file ss_hist_gmd5_pumping.xlsx,
!          sheet RattlesnakeBsn_scenario_1, range a1:n62. A chart at ad3 plots the (negative) pumping within Rattlesnake C basin (col. Q),
!          return flow (positive pumping, col. G) and return flow fraction (col. R, right axis of plot).
!
! Code fixes
! 1. To implement scenarios other than the baseline (see item 8, above), the number of wells on the first line of data for each stress period
!    must be reduced by the number of excluded wells. To do this, the data for the stress period must be retained in memory and then
!    written to the output file after all wells in the stress period have been processed.  --7/11/
!--------------------------------------------------------------------------------------------------------------------------------------------
IMPLICIT NONE

integer, parameter :: mxrow=180
integer, parameter :: mxcol=335
! integer, parameter :: nodes=mxrow*mxcol
integer, parameter :: maxwell = 100000   !dimensioning parameter

!data in file UpStreamGrid.csv:
character*7 chrc  ! format: row|col
character*128 chheader

integer ibound(mxcol,mxrow), inGMD5(mxcol,mxrow)
integer dwrbsn(mxcol,mxrow)
integer alluv(mxcol,mxrow)     !grid cell centers within allvial extent (y=1,n=0)
integer rsblQuiv(mxcol,mxrow)  !Rattlesnake basin grid cells below Quivira intake (y=1,n=0)
integer rscall_seg, rsc_seg
integer rsdist(mxcol,mxrow)
integer ialluv(mxcol,mxrow)     !alluvial cell indicator for cells in Rattlesnake C basin
integer RapRsp1(mxcol,mxrow)    !cells in Rattlesnake C basin with stream depletion fraction > 70 pct (from a Balleau report figure)
integer RapRsp2(mxcol,mxrow)    !cells in Rattlesnake C basin with 40-70pct stream depletion fraction (from a Balleau report figure)
integer RapRsp3(mxcol,mxrow)    !cells in Rattlesnake C basin with 20-40pct stream depletion fraction (from a Balleau report figure)
integer RapRsp4(mxcol,mxrow)    !cells in Rattlesnake C basin with stream depletion fraction < 20 pct (from a Balleau report figure)
integer sfr_RR1(mxcol,mxrow)    !cells in Rattlesnake C basin based on Balleau sfr_10y2.shp with stream depletion fraction 70-100 pct (from Balleau report figure 51)
integer sfr_RR2(mxcol,mxrow)    !cells in Rattlesnake C basin based on Balleau sfr_10y2.shp with stream depletion fraction 40-100 pct (from Balleau report figure 51)
integer sfr_RR3(mxcol,mxrow)    !cells in Rattlesnake C basin based on Balleau sfr_10y2.shp with stream depletion fraction 20-100 pct (from Balleau report figure 51)
integer Mack_1mi(mxcol,mxrow)   !cells in Rattlesnake C basin within 1 mi of Rattlesnake C cutting off at Macksville station
integer Mack_2mi(mxcol,mxrow)   !cells in Rattlesnake C basin within 2 mi of Rattlesnake C cutting off at Macksville station
real*8 surfelev(mxcol,mxrow)

real*8 Q, Q_acft
real*8 strper, sqft_acre, cfd_acft, sec_day
real*8 Qacftyr(mxcol,mxrow), Qacftyr_1957(mxcol,mxrow)
real*8 sumPos, sumNeg, sumAll, sumIncl, sumExcl
real*8 sumGMD5, sumGMD5_jr,sumRsnake,sumRsnake_jr
real*8 sumPosGMD5, sumPosRsnake
integer nwell_Rsnake

integer iyroff     !shutoff year specified for scenarios
real*8 pumpfactor  !factor to be applied to pumping within masked zone

integer nwell_excl ! no. excluded wells in stress period
integer nwell_mod  ! modified number of wells = nwell - nwell_excl
integer idxwell                  !counter for list of wells in stress period
integer locwell(3,maxwell)     !location (lay,row,col) for each well
real*8 Qwell(maxwell)
real*8 afwell(maxwell), sumAug

integer id_scenario,incl_well
integer nrow,ncol,ir,ic,node,iyr,mo
integer irow,icol
real*8 x_ft,y_ft

character*128 ch_scenario
character*2 chmo(12)
character*7 chper

CHARACTER*80 BUFF,WELIN,georeference,WELOUT,WELOUT2,WELSUM
INTEGER I,J,NWELL,LAY,ROW,COL,MXWELL

character*128 FILE
integer IARGC,IOS

!declarations for added wells (augmentation scenarios)
integer, parameter :: mxaug = 10         !max. no. wells to aug
integer numaug, idx
integer layaug(mxaug),rowaug(mxaug),colaug(mxaug)
integer monaug(12)          !monthly augmentation pattern (1 or 0)
real*8  augflow(1939:2007)  !augmentation flow each year (cfs)
real*8  augfraction(mxaug)  !fraction of augmentation pumping from each well
real*8  Qaug(mxaug)         !pumping rate from each well (cfs): product of augflow in a given year and augfraction for a given well

data chmo /'01','02','03','04','05','06','07','08','09','10','11','12'/
!--------------------------------------------------------------------
nrow=mxrow
ncol=mxcol
strper = 30.4375  !stress period length (days); 3 steps per stress period
sec_day = 86400.
sqft_acre = 43560.
cfd_acft = strper/sqft_acre !multiply by Q (cu.ft/day) to convert to ac-ft in stress period

if (IARGC().ne.1) then
  write(*,*) 'Usage: OneLayerWell_Scenarios <parameter file>'
  STOP
endif
call GETARG(1,FILE)
write(*,*) 'Open '//trim(FILE)
OPEN(1,FILE=FILE,STATUS='OLD',IOSTAT=IOS)
if (IOS.ne.0) then
  write(*,*) 'Error opening '//trim(FILE)
  STOP
endif

READ(1,*) WELIN
READ(1,*) georeference
READ(1,*) WELOUT
READ(1,*) WELOUT2
READ(1,*) WELSUM
READ(1,*) MXWELL
read(1,*) id_scenario,ch_scenario
read(1,*) iyrOff,pumpfactor
read(1,*) numaug

monaug=0
augflow=0
augfraction=0

if (numaug.gt.0) then
!   define monthly and annual augmentation pumping patterns:
  read(1,*) (monaug(j),j=1,12)          !1 or 0 (on or off)
  read(1,*) (augflow(j),j=1939,2007)    !cfs
!   define location and fraction of aug pumping contributed by each well:
  do idx=1,numaug
    read(1,*) layaug(idx),rowaug(idx),colaug(idx),augfraction(idx)
  enddo
endif
CLOSE(1)


! input parameter file definitions
! iyrOff: year of shutoff (or reduction); pumpfactor: factor to be applied to wells within masked zone
! numaug: number of wells to be added
! monaug: monthly aug pumping pattern (1:on, 0:off) for calendar months 1-12
! augflow: aug flow rate in each year 1939-2007 (cfs)
! for each well added (numaug > 0):
!   layer,row,col,contributed fraction of aug pumping 

write(*,*) 'mxwell = ',mxwell
write(*,*) 'Scenario ',id_scenario,': '//trim(ch_scenario)

! Read comma-delimited text file UpStreamGrid_CellCenters_Ident.csv for entire upstreamGrid extent (180 rows by 335 columns)
! For each grid cell, read:
!    Node number and associated (row,column) indices;
!    Grid cell center (x,y) coordinates of model projection (KS State Plane South, NAD 1983, U.S. feet);
!    surfelev: Land surface elevation (ft);
!    Ibound:   array to define active model domain (1=active, 0=inactive; -1=specified heads);
!    ingmd5:   GMD5 zone identifier (1 within GMD5, 0 outside GMD5)
!    dwrbsn:   DWR basin identifier (50 for Rattlesnake C)
!    alluv:    alluvial extent identifier (y=1,n=0)
!    rsblQuiv: Rattlesnake basin cells below Quivira intake (y=1,n=0); associated wells exempt from junior rights shutoff
!    rscall_seg: all grid cells whose centers lie within Rattlesnake basin (y=1,n=0)
!    rsc_seg:    all stream cells within Rattlesnake C basin, including Rattlesnake C and its tributaries (y=1,n=0).
!    rsdist:     all grid cells whose centers lie within 1 mi of stream cells in Rattlesnake C basin (y=1,n=0).
!    ialluv:     indicates alluvial cells in Rattlesnake C basin
!    RapRsp[1-4]:"rapid response" indicator for cells in Rattlesnake C basin based on interpretation of a Balleau report figure
!                1:70-100%, 2:40-70%, 3:20-40%, 4:0-20% [10-yr stream depletion fraction of pumping 2020-2030]
!    RR[1-2]:         "rapid response" indicator for cells in Rattlesnake C basin based on Balleau sfr_10y2.shp from report figure 51
!                1:70-100%, 2:40-70% [10-yr stream depletion fraction of pumping 2020-2030]
!-----------------------------------------------------------------------
!        1         2         3         4         5         6         7 2
open(1, file = trim(georeference))
read(1,*) chheader
do irow=1,nrow
    do icol=1,ncol
        read(1,*) node,ir,ic,chrc,x_ft,y_ft,surfelev(ic,ir),ibound(ic,ir),     &
                  ingmd5(ic,ir),dwrbsn(ic,ir),alluv(ic,ir),rsblQuiv(ic,ir),    &
                  rscall_seg,rsc_seg,rsdist(ic,ir),ialluv(ic,ir),              &
                  RapRsp1(ic,ir),RapRsp2(ic,ir),RapRsp3(ic,ir),RapRsp4(ic,ir), &
                  sfr_RR1(ic,ir),sfr_RR2(ic,ir),sfr_RR3(ic,ir),Mack_1mi(ic,ir), &
                  Mack_2mi(ic,ir)
    enddo
enddo
CLOSE(1)

OPEN(21, FILE=WELIN) !input
OPEN(22, FILE=WELOUT) !output
open(23,file=welout2)
open(24,file=welsum)

DO I= 1,4
    READ(21,'(a50)') BUFF
    WRITE(22,'(a50)') BUFF
END DO
DO I=1,MXWELL
    READ(21,*) LAY,ROW,COL,Q
    LAY = 1                                !set LAY=1 ONLY for single-layer model version.
    WRITE(22,'(3I4,F20.5)') LAY,ROW,COL,Q
END DO

iyr=1939
mo=11
write(*,*) 'i  iyr   sumPosGMD5        sumGMD5           sumGMD5_jr    nwell_Rsnake' &
      //'            sumPosRsnake          sumRsnake             sumRsnake_jr' &
      //'          sumPos            sumNeg             sumAll              sumIncl' &
      //'           sumExcl            sumAug'
DO I=1,817

!     Increment modulo month counter mo; on mo=1 increment year and initialize annual sums.
    mo=mod(mo,12)+1
    if(mo.eq.1) then
        iyr=iyr+1
        Qacftyr=0.  !initialize annual pumping array for each grid cell
        sumPos=0
        sumNeg=0
                    !initialize total pumping sums (acre-feet) within GMD5 and within Rattlesnake basin for calendar year
        sumGMD5=0
        sumPosGMD5=0
        sumGMD5_jr=0
        sumRsnake=0
        sumPosRsnake=0
        sumRsnake_jr=0
        sumIncl=0
        sumAug=0
        if (numaug.gt.0 .and. ABS(augflow(iyr)).gt.1d-6) then
          do idx=1,numaug
            Qaug(idx)=sec_day*augflow(iyr)*augfraction(idx) !convert flow from cfs to cfd
          enddo
        endif
    endif
    write(chper,'(a2,1h/,i4)') chmo(mo),iyr
    nwell_Rsnake = 0

    READ(21,*) NWELL,BUFF
    nwell_mod = nwell
    nwell_excl = 0
    idxwell=0       !initialize counter for list of wells to be included in well file
    IF(NWELL .GT. 0) THEN
        DO J=1,NWELL
            READ(21,*) LAY,ROW,COL,Q
            LAY = 1                                !set LAY=1 ONLY for single-layer model version.
            Q_acft = cfd_acft*Q
            Qacftyr(col,row) = Qacftyr(col,row) + Q_acft ! volume into aquifer (out is negative)
            incl_well = 1  !baseline option to include all wells

            if (Q.lt.0) then
              sumNeg = sumNeg + Q_acft         !volume extracted
            elseif (Q.gt.0) then
              sumPos = sumPos + Q_acft         !volume injected
            endif

            if (ingmd5(col,row).eq.1) then
                sumGMD5=sumgmd5 + Q_acft
                if (Q.gt.0) sumPosGMD5 = sumPosGMD5 + Q_acft
                if (iyr.gt.1957.and.abs(Qacftyr_1957(col,row)).lt.1.) then
                    sumGMD5_jr=sumgmd5_jr + Q_acft
                endif
            endif

! implement scenarios here by setting incl_well equal to zero to screen junior pumpers subject to scenario conditions:

            if (dwrbsn(col,row).eq.50) then
                nwell_Rsnake = nwell_Rsnake + 1
                sumRsnake = sumRsnake + Q_acft
                if (Q.gt.0) sumPosRsnake = sumPosRsnake + Q_acft
                if (iyr.gt.1957.and.abs(Qacftyr_1957(col,row)).lt.1.) then
                    sumRsnake_jr=sumRsnake_jr + Q_acft
                    if (id_scenario.eq.1) then
                        if (rsblQuiv(col,row).eq.0) incl_well = 0    ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.2) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0) incl_well = 0  ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.3) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. rsdist(col,row).eq.1) incl_well = 0  ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.4) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. ialluv(col,row).eq.1) incl_well = 0  ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.5) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. RapRsp1(col,row).eq.1) incl_well = 0 ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.6) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. RapRsp2(col,row).eq.1 .and. RapRsp1(col,row).eq.0) incl_well = 0 ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.7) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. sfr_RR1(col,row).eq.1) incl_well = 0 ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.8) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. sfr_RR2(col,row).eq.1) incl_well = 0 ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.9) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. sfr_RR3(col,row).eq.1) incl_well = 0 ! exclude or reduce well (rights bl Quivira intake are exempt)
                    elseif (id_scenario.eq.10) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. Mack_1mi(col,row).eq.1) incl_well = 0 ! exclude or reduce well (rights bl Quirvira intake are exempt)
                    elseif (id_scenario.eq.11) then
                        if (iyr.ge.iyroff .and. rsblQuiv(col,row).eq.0 .and. Mack_2mi(col,row).eq.1) incl_well = 0 ! exclude or reduce well (rights bl QUivira intake are exempt)
                    endif
                endif
            endif

            if (incl_well.eq.1) then
                idxwell = idxwell + 1
                locwell(1,idxwell) = lay
                locwell(2,idxwell) = row
                locwell(3,idxwell) = col
                Qwell(idxwell) = Q
                afwell(idxwell) = Q_acft
                sumIncl = sumIncl + Q_acft

            elseif (pumpfactor.gt.1.d-3) then
                idxwell = idxwell + 1
                locwell(1,idxwell) = lay
                locwell(2,idxwell) = row
                locwell(3,idxwell) = col
                Qwell(idxwell) = pumpfactor*Q
                afwell(idxwell) = pumpfactor*Q_acft
                sumIncl = sumIncl + pumpfactor*Q_acft
            endif

        END DO

        if (numaug.gt.0 .and. monaug(mo).eq.1 .and. ABS(augflow(iyr)).gt.1d-6) then
          do idx=1,numaug
            idxwell = idxwell + 1
            locwell(1,idxwell) = layaug(idx)
            locwell(2,idxwell) = rowaug(idx)
            locwell(3,idxwell) = colaug(idx)
            Qwell(idxwell) = Qaug(idx)
            afwell(idxwell) = cfd_acft*Qaug(idx)
            sumAug = sumAug + afwell(idxwell)
          enddo
        endif

        nwell_mod = idxwell
        nwell_excl = nwell - nwell_mod

        if(mo.eq.12) then
            sumAll = sumPos + sumNeg
            sumExcl = sumAll - sumIncl
            write(*,*) i,iyr,sumPosGMD5,sumGMD5,sumGMD5_jr,nwell_Rsnake,sumPosRsnake,sumRsnake,sumRsnake_jr,sumPos,sumNeg,sumAll,sumIncl,sumExcl,sumAug
            if(iyr.eq.1957) Qacftyr_1957 = Qacftyr
        endif
    ENDIF
    WRITE(22,'(I5,X,A50,1x,a7)')NWELL_mod,BUFF,chper
    WRITE(23,'(I5,X,A50,1x,a7)')NWELL_mod,BUFF,chper
    write(24,'(3i5,1x,a)') nwell, nwell_mod, nwell_excl
    IF(NWELL .GT. 0) THEN
        DO idxwell=1,NWELL_mod
            lay = locwell(1,idxwell)
            row = locwell(2,idxwell)
            col = locwell(3,idxwell)
            Q = Qwell(idxwell)
            Q_acft = afwell(idxwell)
            WRITE(22,'(3I4,ES14.6)') lay,row,col,Q
            if (dwrbsn(col,row).eq.50) WRITE(23,'(3I4,ES14.6,f12.2)') LAY,ROW,COL,Q,Q_acft
        ENDDO
    ENDIF
    READ(21,*) BUFF
    WRITE(22,*) BUFF
END DO 

CLOSE(21)
CLOSE(22)

end program
